##############################
### ROZDZIAL 6 ###############
##############################
### Model wielopoziomowy #####
##############################
### (C) Magdalena Karska #####
##############################

# Importujemy dane
setwd("C:/zastosowania_ekonometrii_www/06")
mdata <- read.csv2("rozdzial_6.csv", header = TRUE)
options(scipen = 4)
#Instalujemy pakiety do analizy wielopoziomowej
install.packages("lme4")
library(lme4)
install.packages("lmerTest")
library(lmerTest)

#Sprawdmy na wykresie czy redni poziom zaufania spoecznego rni si midzy krajami
trust_by_cntry <- sort(tapply(mdata$trust, mdata$cntry, mean))
barplot(trust_by_cntry, col = "dodgerblue3", ylab = "redni poziom zaufania spoecznego w kraju" )


#Centrujemy zmienne
wellbi_c <- scale(mdata$wellbi, center=TRUE, scale = FALSE)
eduyrs_c <- scale(mdata$eduyrs, center=TRUE, scale = FALSE)
sclmeet_c <- scale(mdata$sclmeet, center=TRUE, scale = FALSE)
aesfdrk_c <- scale(mdata$aesfdrk, center=TRUE, scale = FALSE)
protestant_c <- scale(mdata$protestant, center=TRUE, scale = FALSE)
governance_c <- scale(mdata$governance, center=TRUE, scale = FALSE)
inequality_c <- scale(mdata$inequality, center=TRUE, scale = FALSE)

# Krok 1: Budowa modelu zerowego
m1.1<-lmer(trust ~ (1 | cntry), weights = pweight * dweight, mdata, REML = FALSE)
summary(m1.1)

#Krok 2:  Uwzgldnienie staych wspczynnikw regresji dla poziomu jednostek
m1.2<-lmer(trust ~ eduyrs_c + wellbi_c + sclmeet_c + aesfdrk_c +(1 | cntry), weights = pweight * dweight, mdata, REML = FALSE)
summary(m1.2)
anova(m1.1, m1.2)


#Tworzymy wykres dla zmiennej aesfdrk_c
#"Wycigamy" oszacowania parametrw z modelu 
m1.2.coef <- coef(m1.2)
# Na osi x zaznaczymy dwie przykadowe wartoci, ktre moe przyj zmienna aesfdrk_c
x <- c(-1, 2)
# Na osi y zaznaczymy wartoci, ktre przyjmie zmienna objaniana trust,
# dla punktw z osi x dla 19 krajw (przy zaoeniu, e wartoci pozostaych zmiennych s rwne 0)
y <- matrix(nrow = 19, ncol = 2)
rownames(y) <- rownames(m1.2.coef[[1]])
# Obliczmy te wartoci (pamitajmy o dodaniu wyrazw wolnych)
y[, 1] <- m1.2.coef[[1]][,1] + m1.2.coef[[1]][, 5] * (-1)
y[, 2] <- m1.2.coef[[1]][,1] + m1.2.coef[[1]][, 5] * 2

#Naniemy nasze punkty na wykres i poczmy je lini
plot(x, y[1,], ylab = "Przewidywane zaufanie spoeczne",
     xlab = "Poczucie zagroenia po zmroku w okolicy miejsca zamieszkania",
     type = "l", ylim = c(-1.5, 2))
 for (i in 1:19) {points(x, y[i,], type = "l", lty = i)}         
 
#Krok 3: Uwzgldnienie wspczynnikw regresji dla poziomu krajw
m1.3<-lmer(trust ~ eduyrs_c + wellbi_c + sclmeet_c + aesfdrk_c +
        protestant_c + governance_c + inequality_c + (1 | cntry),
        weights = pweight * dweight, mdata, REML = FALSE)
summary(m1.3)
anova(m1.2, m1.3)
#ZObaczmy co zmienio si na naszym wykresie, gdy nasze wyrazy wolne
#s objaniane przez zmienne z poziomu krajw
m1.3.coef <- coef(m1.3)
m1.3.coef
x<-cbind(-1, 2)
y <- matrix(nrow = 19,ncol = 2)
rownames(y) <- rownames(m1.3.coef[[1]])

#"Wycignijmy" wartoci zmiennych objaniajcych z poziomu krajw 
# Mog w tym celu uy funkcji max, bo warto jest taka sama dla kadej obserwacji z jednego kraju
protestant_value <- c(tapply(protestant_c, mdata$cntry, max))
governance_value <- c(tapply(governance_c, mdata$cntry, max))
inequality_value <- c(tapply(inequality_c, mdata$cntry, max))

y[,1] <- m1.3.coef[[1]][, 1] + m1.3.coef[[1]][, 5] * (-1) + m1.3.coef[[1]][, 6] * protestant_value +
  + m1.3.coef[[1]][, 7] * governance_value + m1.3.coef[[1]][, 8] * inequality_value
y[,2] <- m1.3.coef[[1]][, 1] + m1.3.coef[[1]][, 5] * (2) + m1.3.coef[[1]][, 6] * protestant_value +
  + m1.3.coef[[1]][, 7] * governance_value + m1.3.coef[[1]][, 8] * inequality_value

plot(x, y[1,], ylab = "Przewidywane zaufanie spoeczne",
     xlab = "Poczucie zagroenia po zmroku w okolicy miejsca zamieszkania",
     type = "l", ylim = c(-1.5, 2))
for (i in 2:19) {points(x, y[i, ], type = "l", lty = i)}


#Krok 4: Uwzgldnienie losowych wspczynnikw regresji
m1.4<-lmer(trust ~ eduyrs_c + wellbi_c + sclmeet_c + aesfdrk_c
             + protestant_c + governance_c + inequality_c + 
            (1+ eduyrs_c + wellbi_c + sclmeet_c + aesfdrk_c |cntry),
            weights = pweight * dweight, mdata, REML = FALSE)
summary(m1.4)
anova(m1.3, m1.4)
#Jak teraz bdzie wyglda nasz wykres?
m1.4.coef <- coef(m1.4)
m1.4.coef
x <- c(-1, 2)
y <- matrix(nrow = 19, ncol = 2)
rownames(y) <- rownames(m1.4.coef[[1]])

y[,1] <- m1.4.coef[[1]][, 1] + m1.4.coef[[1]][, 5] * (-1) + m1.4.coef[[1]][, 6] * protestant_value +
  + m1.4.coef[[1]][, 7] * governance_value + m1.4.coef[[1]][, 8] * inequality_value
y[,2] <- m1.4.coef[[1]][, 1] + m1.4.coef[[1]][, 5] * (2) + m1.4.coef[[1]][, 6] * protestant_value +
  + m1.4.coef[[1]][, 7] * governance_value + m1.4.coef[[1]][, 8] * inequality_value

plot(x, y[1, ], ylab = "Przewidywane zaufanie spoeczne",
     xlab = "Poczucie zagroenia po zmroku w okolicy miejsca zamieszkania",
     type = "l", ylim = c(-1.5, 2))
for (i in 2:19) {points(x, y[i, ], type = "l", lty = i)}

#Krok 5: Uwzgldnienie midzypoziomowych interakcji 
#Dodajemy do modelu interakcj zmiennej z poziomu jednostek wellbi_c 
#ze zmienn z poziomu krajw gini_c
m1.5 <- lmer(trust ~ eduyrs_c + wellbi_c + sclmeet_c + aesfdrk_c 
           + protestant_c + governance_c + inequality_c + wellbi_c * inequality_c + 
          (1 + eduyrs_c + wellbi_c + sclmeet_c + aesfdrk_c | cntry),
          weights = pweight*dweight, mdata, REML = FALSE)
summary(m1.5)
anova(m1.4, m1.5)

#rdo kodu: R Textbook Examples. UCLA: Statistical Consulting Group.
#http://www.ats.ucla.edu/stat/r/examples/mlm_imm/immch3.htm
